@@ -1,7 +1,6 @@
 ;;; font-latex.el --- LaTeX fontification for Font Lock mode.
 
-;; Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-;;   2004, 2005, 2006, 2007, 2008 Free Software Foundation.
+;; Copyright (C) 1996-2014  Free Software Foundation, Inc.
 
 ;; Authors:    Peter S. Galbraith <psg@debian.org>
 ;;             Simon Marshall <Simon.Marshall@esrin.esa.it>
@@ -128,8 +127,7 @@ If nil, quoted content will not be fontified."
 QUOTES has to be a list adhering to the format of an element of
 `font-latex-quote-list'."
   (setq font-latex-quotes-control nil)
-  (make-local-variable 'font-latex-quote-list)
-  (add-to-list 'font-latex-quote-list quotes))
+  (add-to-list (make-local-variable 'font-latex-quote-list) quotes))
 
 (defun font-latex-quotes-set-internal ()
   "Set `font-latex-quotes-internal' according to `font-latex-quotes'.
@@ -183,7 +181,10 @@ correct value from document properties."
   "Update sectioning commands faces."
   (unless height-scale
     (setq height-scale (if (numberp font-latex-fontify-sectioning)
-			   font-latex-fontify-sectioning
+			   ;; Make sure `height-scale' is a floating point
+			   ;; number because `set-face-attribute' treats
+			   ;; integers differently from floating points.
+			   (float font-latex-fontify-sectioning)
 			 1.1)))
   (unless max
     (setq max font-latex-sectioning-max))
@@ -237,7 +238,10 @@ Emacs."
   (unless max (setq max font-latex-sectioning-max))
   (unless height-scale
     (setq height-scale (if (numberp font-latex-fontify-sectioning)
-			   font-latex-fontify-sectioning
+			   ;; Make sure `height-scale' is a floating point
+			   ;; number because the integer type is treated
+			   ;; differently.
+			   (float font-latex-fontify-sectioning)
 			 1.1)))
   (dotimes (num max)
     (let* (;; reverse for XEmacs:
@@ -256,7 +260,7 @@ Emacs."
 
 Probably you don't want to customize this face directly.  Better
 change the base face `font-latex-sectioning-5-face' or customize the
-variable `font-latex-fontify-sectioning'." num)
+variable `font-latex-fontify-sectioning'." ',num)
 	  :group 'font-latex-highlighting-faces))
       (when (and (featurep 'xemacs)
 		 ;; Do not touch customized  faces.
@@ -284,11 +288,43 @@ variable `font-latex-fontify-sectioning'." num)
       "appendix" "displaybreak" "allowdisplaybreaks" "include")
      'font-latex-warning-face 1 noarg)
     ("variable"
-     (("setlength" "|{\\{") ("settowidth" "|{\\{") ("setcounter" "{|{\\")
-      ("addtolength" "|{\\{") ("addtocounter" "{|{\\"))
+     (("setlength" "|{\\{") ("settowidth" "|{\\{") ("settoheight" "{{")
+      ("settodepth" "{{") ("setcounter" "{|{\\")
+      ("addtolength" "|{\\{") ("addtocounter" "{|{\\")
+      ("stepcounter" "{") ("refstepcounter" "{")
+      ("arabic" "{") ("roman" "{") ("Roman" "{") ("alph" "{") ("Alph" "{")
+      ("fnsymbol" "{"))
      'font-lock-variable-name-face 2 command)
+    ("biblatexnoarg"
+     ("newrefsegment" "mancite" "pno" "ppno" "nopp" "psq" "psqq")
+     'font-lock-variable-name-face 2 noarg)
+    ("biblatex"
+     (("newrefsection" "[") ("ExecuteBibliographyOptions" "[{")
+      ("printbibliography" "[") ("printshorthands" "[") ("printbibheading" "[")
+      ("addbibresource" "[{") ("addglobalbib" "[{") ("addsectionbib" "[{")
+      ("bibbysection" "[") ("bibbysegment" "[") ("bibbycategory" "[")
+      ("DeclareBibliographyCategory" "{") ("addtocategory" "{{") ("defbibenvironment" "{{{{")
+      ("defbibheading" "{[{") ("defbibnote" "{{") ("defbibfilter" "{{") ("defbibcheck" "{{")
+      ("defbibentryset" "{{") ("Cite" "[[{") ("parencite" "*[[{") ("Parencite" "[[{")
+      ("footcite" "[[{") ("footcitetext" "[[{") ("textcite" "[[{") ("Textcite" "[[{")
+      ("smartcite" "[[{") ("Smartcite" "[[{") ("supercite" "{") ("autocite" "*[[{")
+      ("Autocite" "*[[{") ("citeauthor" "[[{") ("Citeauthor" "[[{") ("citetitle" "*[[{")
+      ("citeyear" "*[[{") ("citedate" "*[[{") ("citeurl" "[[{") ("parentext" "{")
+      ("brackettext" "{") ("fullcite" "[[{") ("fullfootcite" "[[{") ("volcite" "[{[[")
+      ("Volcite" "[{[[") ("pvolcite" "[{[[") ("Pvolcite" "[{[[") ("fvolcite" "[{[[")
+      ("ftvolcite" "[{[[") ("svolcite" "[{[[") ("Svolcite" "[{[[") ("tvolcite" "[{[[")
+      ("Tvolcite" "[{[[") ("avolcite" "[{[[") ("Avolcite" "[{[[") ("notecite" "[[{")
+      ("Notecite" "[[{") ("pnotecite" "[[{") ("Pnotecite" "[[{") ("fnotecite" "[[{")
+      ("citename" "[[{[{") ("citelist" "[[{[{") ("citefield" "[[{[{") ("citereset" "*")
+      ("RN" "{") ("Rn" "{") ("DefineBibliographyStrings" "{{") ("DefineBibliographyExtras" "{{")
+      ("UndefineBibliographyExtras" "{{") ("DefineHyphenationExceptions" "{{")
+      ("NewBibliographyString" "{") ("autocites" "(([[{") ("Autocites" "(([[{")
+      ("cites" "(([[{") ("Cites" "(([[{") ("parencites" "(([[{") ("Parencites" "(([[{")
+      ("footcites" "(([[{") ("footcitetexts" "(([[{") ("smartcites" "(([[{")
+      ("Smartcites" "(([[{") ("textcites" "(([[{") ("Textcites" "(([[{") ("supercites" "(([[{"))
+     'font-lock-constant-face 2 command)
     ("reference"
-     (("nocite" "{") ("cite" "[{") ("label" "{") ("pageref" "{")
+     (("nocite" "*{") ("cite" "*[[{") ("label" "{") ("pageref" "{")
       ("vref" "{") ("eqref" "{") ("ref" "{") ("include" "{")
       ("input" "{") ("bibliography" "{") ("index" "{") ("glossary" "{")
       ("footnote" "[{") ("footnotemark" "[") ("footnotetext" "[{"))
@@ -297,12 +333,13 @@ variable `font-latex-fontify-sectioning'." num)
      (("begin" "{") ("end" "{") ("pagenumbering" "{")
       ("thispagestyle" "{") ("pagestyle" "{") ("nofiles" "")
       ("includeonly" "{") ("bibliographystyle" "{") ("documentstyle" "[{")
-      ("documentclass" "[{") ("newenvironment" "*{[[{{")
+      ("documentclass" "[{[") ("newenvironment" "*{[[{{")
       ("newcommand" "*|{\\[[{") ("newlength" "|{\\")
       ("newtheorem" "{[{[")
-      ("newcounter" "{[") ("renewenvironment" "*{[{{")
+      ("providecommand" "*|{\\[[{")
+      ("newcounter" "{[") ("renewenvironment" "*{[[{{")
       ("renewcommand" "*|{\\[[{") ("renewtheorem" "{[{[")
-      ("usepackage" "[{") ("fbox" "{") ("mbox" "{") ("sbox" "{")
+      ("usepackage" "[{[") ("fbox" "{") ("mbox" "{") ("rule" "[{{")
       ("vspace" "*{") ("hspace" "*{") ("thinspace" "") ("negthinspace" "")
       ;; XXX: Should macros without arguments rather be listed in a
       ;; separate category with 'noarg instead of 'command handling?
@@ -446,7 +483,11 @@ and `font-latex-make-user-keywords' and not intended for general
 use."
   ;; Note: The functions are byte-compiled at the end of font-latex.el.
   ;; FIXME: Is the cond-clause possible inside of the defun?
-  (cond ((eq type 'command)
+
+  ;; In an earlier version of font-latex the type could be a list like
+  ;; (command 1).  This indicated a macro with one argument.  Provide
+  ;; a match function in this case but don't actually support it.
+  (cond ((or (eq type 'command) (listp type))
 	 (eval `(defun ,(intern (concat prefix name)) (limit)
 		  ,(concat "Fontify `" prefix name "' up to LIMIT.
 
@@ -494,7 +535,10 @@ construct to be highlighted.  Currently the symbols 'command,
 This is a helper function for `font-latex-make-built-in-keywords'
 and `font-latex-make-user-keywords' and not intended for general
 use."
-  (cond ((eq type 'command)
+  ;; In an earlier version of font-latex the type could be a list like
+  ;; (command 1).  This indicated a macro with one argument.  Provide
+  ;; a matcher in this case but don't actually support it.
+  (cond ((or (eq type 'command) (listp type))
 	 `(,(intern (concat prefix name))
 	   (0 (font-latex-matched-face 0) append t)
 	   (1 (font-latex-matched-face 1) append t)
@@ -506,7 +550,10 @@ use."
 	   (7 (font-latex-matched-face 7) append t)))
 	((eq type 'noarg)
 	 `(,(intern (concat prefix name))
-	   (0 ,face)))
+	   ;; Quote a list of face properties but do not to quote a face symbol.
+	   (0 ,(if (and (listp face) (not (fboundp (car face))))
+		   `',face
+		 face))))
 	((eq type 'declaration)
 	 `(,(intern (concat prefix name))
 	   (0 'font-latex-warning-face t t)
@@ -528,20 +575,19 @@ use."
 	       ',keywords
 	       ,(concat "Buffer-local keywords to add to `"
 			prefix name "-keywords'.
-This must be a list of keyword strings \(not regular expressions\) omitting
-the leading backslash.  It will get transformed into a regexp using
-`" prefix name "-make'.  This variable is not for end users; they
-should customize `" prefix name "-keywords' instead.  It is for
-authors of Lisp files that get loaded when LaTeX style files are used in the
-current buffer.  They should add keywords to this list and rebuild the
-fontification regexp like so:
 
- (add-to-list '" prefix name "-keywords-local \"setstuff\")
- (" prefix name "-make)
+This must be a list where each element is a list consisting of a
+keyword string \(not a regular expression\) omitting the leading
+backslash and a format specifier as.  The options for the format
+specifier are described in the doc string of
+`font-latex-user-keyword-classes'.
+
+This is an internal variable which should not be set directly.
+Use `font-latex-add-keywords' instead.
 
 Generated by `font-latex-make-built-in-keywords'.")))
-      (eval `(make-variable-buffer-local
-	      ',(intern (concat prefix name "-keywords-local"))))
+      (make-variable-buffer-local
+       (intern (concat prefix name "-keywords-local")))
 
       ;; defun font-latex-match-*-make
       ;; Note: The functions are byte-compiled at the end of font-latex.el.
@@ -559,8 +605,8 @@ Generated by `font-latex-make-built-in-keywords'.")
 		 (dolist (elt keywords)
 		   (let ((keyword (if (listp elt) (car elt) elt)))
 		     (if (string-match "^[A-Za-z]" keyword)
-			 (add-to-list 'multi-char-macros keyword)
-		       (add-to-list 'single-char-macros keyword))))
+			 (push keyword multi-char-macros)
+		       (push keyword single-char-macros))))
 		 (when (or multi-char-macros single-char-macros)
 		   (setq ,(intern (concat prefix name))
 			 (concat
@@ -576,15 +622,27 @@ Generated by `font-latex-make-built-in-keywords'.")
 
       ;; defcustom font-latex-match-*-keywords
       (eval `(defcustom ,(intern (concat prefix name "-keywords")) nil
-	       ,(concat "List of keyword strings for " name " face.
-Each string has to be the name of a macro omitting the leading backslash.
-
-Setting this variable directly does not take effect;
-restart Emacs.
+	       ,(concat "List of keywords "
+			(when (eq type 'command) "and formats ")
+			"for " name " face.\n"
+			(if (eq type 'command)
+			    "\
+Each element has to be a list consisting of the name of a macro
+omitting the leading backslash and a format specifier as
+described in the doc string of `font-latex-user-keyword-classes'."
+			  "\
+Each element has to be the name of a macro as a string, omitting
+the leading backslash.")
+			"\n\n\
+Setting this variable directly does not take effect; restart
+Emacs.
 
 Generated by `font-latex-make-built-in-keywords'.")
-	       :type '(repeat (list (string :tag "Keyword")
-				    (string :tag "Format")))
+	       :type '(repeat (string :tag "Keyword"))
+	       :type '(repeat ,(if (eq type 'command)
+				   '(list (string :tag "Keyword")
+					  (string :tag "Format"))
+				 '(string :tag "Keyword")))
 	       :set (lambda (symbol value)
 		      (set-default symbol value)
 		      (funcall ',(intern (concat prefix name "-make"))))
@@ -593,8 +651,7 @@ Generated by `font-latex-make-built-in-keywords'.")
       ;; defvar font-latex-match-*
       (eval `(defvar ,(intern (concat prefix name))
 	       ,(intern (concat prefix name "-keywords"))))
-      (eval `(make-variable-buffer-local
-	      ',(intern (concat prefix name))))
+      (make-variable-buffer-local (intern (concat prefix name)))
 
       ;; defun font-latex-match-*
       (font-latex-make-match-defun prefix name face type)
@@ -752,7 +809,7 @@ Generated by `font-latex-make-user-keywords'.")))
 	      (1 (get-text-property (match-end 1) 'face) t))))
     (add-to-list 'font-latex-keywords-1 item)
     (add-to-list 'font-latex-keywords-2 item))
-  (dolist (item 
+  (dolist (item
 	   '((font-latex-match-math-env
 	      (0 'font-latex-warning-face t t)
 	      (1 'font-latex-math-face append t))
@@ -769,11 +826,11 @@ Generated by `font-latex-make-user-keywords'.")))
   "Add KEYWORDS to CLASS.
 KEYWORDS is a list of keywords or keywords with syntax specs.
 CLASS corresponds to a keyword class and can be one of the
-symbols 'warning, 'variable, 'reference, 'function, sectioning-0,
-'sectioning-1, 'sectioning-2, 'sectioning-3, 'sectioning-4,
-'sectioning-5, 'slide-title, 'textual, 'bold-command,
-'italic-command, 'math-command, 'type-command, 'bold-declaration,
-'italic-declaration or 'type-declaration.
+symbols 'warning, 'variable, 'reference, 'biblatex, 'function,
+'sectioning-0, 'sectioning-1, 'sectioning-2, 'sectioning-3,
+'sectioning-4, 'sectioning-5, 'slide-title, 'textual,
+'bold-command, 'italic-command, 'math-command, 'type-command,
+'bold-declaration, 'italic-declaration or 'type-declaration.
 
 The keywords will be added to the buffer-local list of keywords
 of the respective keyword class and necessary updates of the font
@@ -820,6 +877,9 @@ The car is used for subscript, the cdr is used for superscripts."
 The form should be the same as in `font-lock-syntactic-keywords'.")
 (make-variable-buffer-local 'font-latex-syntactic-keywords-extra)
 
+;; Set and updated in `font-latex-set-syntactic-keywords'.
+(defvar font-latex-doctex-syntactic-keywords nil)
+
 (defun font-latex-set-syntactic-keywords ()
   "Set the variable `font-latex-syntactic-keywords'.
 This function can be used to refresh the variable in case other
@@ -843,8 +903,25 @@ have changed."
 	  font-latex-syntactic-keywords nil)
     (unless (= (length verb-envs) 0)
       (add-to-list 'font-latex-syntactic-keywords
-		   `(,(concat "^[ \t]*\\\\begin *{\\(?:" verb-envs
-			      "\\)}.*\\(\n\\)")
+		   `(,(concat
+		       "^[ \t]*\\\\begin *{\\(?:" verb-envs "\\)}"
+		       ;; Some environments accept an optional and/or mandatory
+		       ;; argument that can span over more lines.  Between
+		       ;; "\begin{<envname>}" and the optional argument there
+		       ;; can be whitespaces and the newline can be commented
+		       ;; by a "%" character.
+		       "[ \t]*\\(?:%.*\n[ \t]*\\)?"
+		       ;; The following line of the regexp matches the optional
+		       ;; argument and allows for up to one level of brackets
+		       ;; inside the argument (e.g., the dialect of a language
+		       ;; in the `lstlisting' environment by the `listings'
+		       ;; package).
+		       "\\(?:\\[[^\]\[]*\\(?:\\[[^\]\[]*\\][^\]\[]*\\)*\\]\\)?"
+		       ;; After the optional argument, there may also be
+		       ;; another mandatory argument (e.g. with VerbatimOut or
+		       ;; the minted envs).
+		       "\\(?:{[^}]+}\\)?"
+		       "\\(\n\\)")
 		     (1 "|" t)))
       (add-to-list 'font-latex-syntactic-keywords
 		   ;; Using the newline character for the syntax
@@ -1059,6 +1136,25 @@ returned.
 All specified functions will be called and the region extended
 backwards to the minimum over their return values.")
 
+(defvar font-latex-syntax-alist
+  ;; Use word syntax for @ because we use \> for matching macros and
+  ;; we don't want \foo@bar to be found if we search for \foo.
+  '((?\( . ".") (?\) . ".") (?$ . "\"") (?@ . "w"))
+  "List of specifiers for the syntax alist of `font-lock-defaults'.")
+
+(defun font-latex-add-to-syntax-alist (list)
+  "Activate syntactic font locking for the entries in LIST.
+The entries are added to `font-latex-syntax-alist' and eventually
+end up in `font-lock-defaults'.  Each entry in LIST should be a
+cons pair as expected by `font-lock-defaults'.  The function also
+triggers Font Lock to recognize the change."
+  (make-local-variable 'font-latex-syntax-alist)
+  (set (make-local-variable 'font-latex-syntax-alist)
+       (append font-latex-syntax-alist list))
+  ;; Tell font-lock about the update.
+  (setq font-lock-set-defaults nil)
+  (font-latex-setup))
+
 ;;;###autoload
 (defun font-latex-setup ()
   "Setup this buffer for LaTeX font-lock.  Usually called from a hook."
@@ -1068,17 +1164,17 @@ backwards to the minimum over their return values.")
   (when (fboundp 'built-in-face-specifiers)
     ;; Cool patch from Christoph Wedler...
     (let (instance)
-      (mapcar (lambda (property)
-		(setq instance
-		      (face-property-instance 'font-latex-math-face property
-					      nil 0 t))
-		(if (numberp instance)
-		    (setq instance
-			  (face-property-instance 'default property nil 0)))
-		(or (numberp instance)
-		    (set-face-property 'font-lock-string-face property
-				       instance (current-buffer))))
-	      (built-in-face-specifiers))))
+      (mapc (lambda (property)
+	      (setq instance
+		    (face-property-instance 'font-latex-math-face property
+					    nil 0 t))
+	      (if (numberp instance)
+		  (setq instance
+			(face-property-instance 'default property nil 0)))
+	      (or (numberp instance)
+		  (set-face-property 'font-lock-string-face property
+				     instance (current-buffer))))
+	    (built-in-face-specifiers))))
 
   ;; Activate multi-line fontification facilities if available.
   (when (boundp 'font-lock-multiline)
@@ -1101,10 +1197,8 @@ backwards to the minimum over their return values.")
   ;; `VirTeX-common-initialization' and place it in the different
   ;; `xxx-mode' calls instead, but _after_ `major-mode' is set.
   (let ((defaults
-	 '((font-latex-keywords font-latex-keywords-1 font-latex-keywords-2)
-	   ;; Use word syntax for @ because we use \> for matching macros
-	   ;; and we don't want \foo@bar to be found if we search for \foo.
-	   nil nil ((?\( . ".") (?\) . ".") (?$ . "\"") (?@ . "w")) nil))
+	 `((font-latex-keywords font-latex-keywords-1 font-latex-keywords-2)
+	   nil nil ,font-latex-syntax-alist nil))
 	(variables
 	 '((font-lock-comment-start-regexp . "%")
 	   (font-lock-mark-block-function . mark-paragraph)
@@ -1134,8 +1228,8 @@ backwards to the minimum over their return values.")
 	(progn
 	  ;; XEmacs does not set these variables via `font-lock-defaults'
 	  ;; but requires them to be set explicitely.
-	  (mapcar (lambda (alist)
-		    (set (car alist) (cdr alist))) variables)
+	  (mapc (lambda (alist)
+		  (set (car alist) (cdr alist))) variables)
 	  ;; Has to be set to t as otherwise syntax properties will not be
 	  ;; be picked up during fontification.
 	  (set (make-local-variable 'lookup-syntax-properties) t))
@@ -1145,8 +1239,6 @@ backwards to the minimum over their return values.")
 
 (defun font-latex-jit-lock-force-redisplay (buf start end)
   "Compatibility for Emacsen not offering `jit-lock-force-redisplay'."
-  (if (fboundp 'jit-lock-force-redisplay)
-      (jit-lock-force-redisplay buf start end)
     ;; The following block is an expansion of `jit-lock-force-redisplay'
     ;; and involved macros taken from CVS Emacs on 2007-04-28.
     (with-current-buffer buf
@@ -1161,7 +1253,7 @@ backwards to the minimum over their return values.")
 		  buffer-file-truename)
 	      (put-text-property start end 'fontified t))
 	  (unless modified
-	    (restore-buffer-modified-p nil)))))))
+	    (restore-buffer-modified-p nil))))))
 
 (defun font-latex-fontify-region (beg end &optional loudly)
   "Fontify region from BEG to END.
@@ -1247,16 +1339,7 @@ ignored during the search."
      (condition-case nil
 	 (progn
 	   (goto-char (with-syntax-table
-			  (let ((table (TeX-search-syntax-table)))
-			    ;; Give `openchar' and `closechar' open paren and
-			    ;; close paren syntax respectively.
-			    (modify-syntax-entry
-			     openchar (concat "(" (char-to-string closechar))
-			     table)
-			    (modify-syntax-entry
-			     closechar (concat ")" (char-to-string openchar))
-			     table)
-			    table)
+			  (TeX-search-syntax-table openchar closechar)
 			(scan-sexps (point) 1)))
 	   ;; No error code.  See if closechar is unquoted
 	   (save-excursion
@@ -1375,13 +1458,13 @@ Returns nil if none of KEYWORDS is found."
 					  (match-beginning 0))
 	(let* ((beg (match-beginning 0))
 	       end		   ; Used for multiline text property.
-	       match-data match-beg syntax-error alternative spec
+	       (match-data (list beg))
+	       match-beg syntax-error alternative spec
 	       error-indicator-pos
 	       (spec-list (string-to-list
 			   (or (cadr (assoc (match-string 1) keywords))
 			       font-latex-command-with-args-default-spec)))
 	       (parse-sexp-ignore-comments t)) ; scan-sexps ignores comments
-	  (add-to-list 'match-data beg)
 	  (goto-char (match-end 0))
 	  ;; Check for starred macro if first spec is an asterisk.
 	  (when (eq (car spec-list) ?*)
@@ -1389,7 +1472,7 @@ Returns nil if none of KEYWORDS is found."
 	    (skip-chars-forward "*" (1+ (point))))
 	  ;; Add current point to match data and use keyword face for
 	  ;; region from start to point.
-	  (add-to-list 'match-data (point) t)
+	  (nconc match-data (list (point)))
 	  (add-to-list 'font-latex-matched-faces 'font-lock-keyword-face)
 	  (setq end (point))
 	  (catch 'break
@@ -1495,7 +1578,7 @@ Returns nil if no command is found."
 					  (match-beginning 0))
 	(let ((kbeg (match-beginning 0)) (kend (match-end 1))
 	      (beg  (match-end 0))
-	      end cbeg cend
+	      cbeg cend
 	      (parse-sexp-ignore-comments t)) ; scan-sexps ignores comments
 	  (goto-char kbeg)
 	  (if (not (eq (preceding-char) ?\{))
@@ -1562,16 +1645,18 @@ Used for patterns like:
     (while (re-search-forward "\\(\\\\(\\)\\|\\(\\\\\\[\\)" limit t)
       (unless (save-excursion
 		(goto-char (match-beginning 0))
-		(eq (preceding-char) ?\\)) ; \\[ is not a math environment
+		;; \\[ does not start a math environment
+		(/= (mod (skip-chars-backward "\\\\") 2) 0))
 	(let ((beg (match-beginning 0))
 	      (open-tag (if (match-beginning 1) "\\(" "\\["))
 	      (close-tag (if (match-beginning 1) "\\)" "\\]")))
 	  ;; Search for both opening and closing tags in order to be
 	  ;; able to avoid erroneously matching stuff like "\(foo \(bar\)".
-	  (if (and (re-search-forward (concat (regexp-quote open-tag) "\\|"
-					      (regexp-quote close-tag))
+	  (if (and (re-search-forward (concat "[^\\]\\(?:\\\\\\\\\\)*\\("
+					      (regexp-quote open-tag) "\\|"
+					      (regexp-quote close-tag) "\\)")
 				      limit 'move)
-		   (string= (match-string 0) close-tag))
+		   (string= (match-string 1) close-tag))
 	      ;; Found closing tag.
 	      (progn
 		(font-latex-put-multiline-property-maybe beg (point))
@@ -1590,15 +1675,20 @@ END marks boundaries for searching for environment ends."
     (goto-char end)
     (catch 'extend
       (while (re-search-backward "\\(\\\\)\\)\\|\\(\\\\]\\)" beg t)
-	(when (and (search-backward (if (match-beginning 1) "\\(" "\\[")
-				    (- beg font-latex-multiline-boundary) t)
+	(when (and (zerop (mod (skip-chars-backward "\\\\") 2))
+		   (re-search-backward
+		    (concat "[^\\]\\(?:\\\\\\\\\\)*\\("
+			    (regexp-quote (if (match-beginning 1) "\\(" "\\["))
+			    "\\)")
+		    (- beg font-latex-multiline-boundary) t)
+		   (goto-char (match-beginning 1))
 		   (< (point) beg))
 	  (throw 'extend (point))))
       nil)))
 
 (defcustom font-latex-math-environments
   '("display" "displaymath" "equation" "eqnarray" "gather" "multline"
-    "align" "alignat" "xalignat")
+    "align" "alignat" "xalignat" "xxalignat" "flalign")
   "List of math environment names for font locking."
   :type '(repeat string)
   :group 'font-latex)
@@ -1843,9 +1933,6 @@ END marks boundaries for searching for quotation ends."
   (append font-latex-keywords-2
 	  '(("^%<[^>]*>" (0 font-latex-doctex-preprocessor-face t)))))
 
-;; Set and updated in `font-latex-set-syntactic-keywords'.
-(defvar font-latex-doctex-syntactic-keywords nil)
-
 ;; Copy and adaptation of `doctex-font-lock-^^A' in `tex-mode.el' of
 ;; CVS Emacs (March 2004)
 (defun font-latex-doctex-^^A ()
@@ -1907,8 +1994,8 @@ END marks boundaries for searching for quotation ends."
        (symbol-function 'font-latex-make-built-in-keywords))
   (dolist (elt font-latex-built-in-keyword-classes)
     (let ((name (nth 0 elt)))
-      (byte-compile (intern (concat "font-latex-" name)))
-      (byte-compile (intern (concat "font-latex-" name "-make"))))))
+      (byte-compile (intern (concat "font-latex-match-" name)))
+      (byte-compile (intern (concat "font-latex-match-" name "-make"))))))
 
 
 ;; Provide ourselves:
