@@ -1,9 +1,9 @@
 ;;; foils.el - Special code for FoilTeX.
 
-;; $Id: foils.el,v 1.3 1999/07/16 13:48:17 abraham Exp $
-
 ;;; Code:
 
+(require 'timezone)
+
 (TeX-add-style-hook "foils"
  (function
   (lambda ()
@@ -11,16 +11,16 @@
     (setq LaTeX-default-style "foils")
     (setq LaTeX-default-options '("landscape"))
     (TeX-add-symbols
-     '("foilhead" [ "Rubric-body separation" ] "Foil rubric")))))
+     '("foilhead" [ "Rubric-body separation" ] "Foil rubric"))))
+ LaTeX-dialect)
 
 (defun LaTeX-style-foils nil
   "Prompt for and insert foiltex options."
-  (require 'timezone)
   (let* ((date (timezone-parse-date (current-time-string)))
-	 (year   (string-to-int (aref date 0)))
-	 (month  (string-to-int (aref date 1)))
-	 (day    (string-to-int (aref date 2)))
-	 (title (read-input "Title: ")))
+	 (year   (string-to-number (aref date 0)))
+	 (month  (string-to-number (aref date 1)))
+	 (day    (string-to-number (aref date 2)))
+	 (title (read-string "Title: ")))
     (save-excursion
       (goto-char (point-max))
       (re-search-backward ".begin.document.")
