@@ -1,7 +1,28 @@
-;;; natbib.el --- Style hook for the NatBib package
-;;;
-;;; AUTHOR: Carsten Dominik <dominik@strw.leidenuniv.nl>
-;;;         building on older code from Berwin Turlach
+;;; natbib.el --- Style hook for the natbib package
+
+;; Copyright (C) 1997, 1998, 2004, 2007 Free Software Foundation, Inc.
+
+;; Authors: Berwin Turlach <statba@nus.edu.sg>
+;;          Carsten Dominik <dominik@strw.leidenuniv.nl>
+;; Maintainer: auctex-devel@gnu.org
+;; Keywords: tex
+
+;; This file is part of AUCTeX.
+
+;; AUCTeX is free software; you can redistribute it and/or modify it
+;; under the terms of the GNU General Public License as published by
+;; the Free Software Foundation; either version 3, or (at your option)
+;; any later version.
+
+;; AUCTeX is distributed in the hope that it will be useful, but
+;; WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;; General Public License for more details.
+
+;; You should have received a copy of the GNU General Public License
+;; along with AUCTeX; see the file COPYING.  If not, write to the Free
+;; Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+;; 02110-1301, USA.
 
 ;;; Code:
 
@@ -68,9 +89,10 @@
 	4 'LaTeX-bibitem-list "}")))
 
     ;; Fontification
-    (when (and (featurep 'font-latex)
+    (when (and (fboundp 'font-latex-add-keywords)
 	       (eq TeX-install-font-lock 'font-latex-setup))
-      (font-latex-add-keywords '(("citet" "*[[{")
+      (font-latex-add-keywords '(("cite" "*[[{")
+				 ("citet" "*[[{")
 				 ("citealt" "*[[{")
 				 ("citep" "*[[{")
 				 ("citealp" "*[[{")
@@ -83,7 +105,8 @@
 
     ;; Tell RefTeX
     (if (fboundp 'reftex-set-cite-format)
-	(reftex-set-cite-format 'natbib)))))
+	(reftex-set-cite-format 'natbib))))
+ LaTeX-dialect)
 
 (defun natbib-note-args (optional &optional prompt definition)
   "Prompt for two note arguments a natbib citation command."
