@@ -1,12 +1,11 @@
 ;;; -*- emacs-lisp -*-
 ;;; scrartcl.el -- AUCTeX style for scrartcl.cls
 
-;; Copyright (C) 2002 Free Software Foundation
+;; Copyright (C) 2002, 2005 Free Software Foundation
 ;; License: GPL, see the file COPYING in the base directory of AUCTeX
 
 ;; Author: Mark Trettin <Mark.Trettin@gmx.de>
 ;; Created: 2002-09-26
-;; Version: $Id: scrartcl.el,v 1.3 2004/08/15 20:11:21 dak Exp $
 ;; Keywords: tex
 
 ;;; Commentary:
@@ -19,8 +18,9 @@
 ;;; Code:
 (TeX-add-style-hook "scrartcl"
    (lambda ()
-     (setq LaTeX-largest-level (LaTeX-section-level "section"))
+     (LaTeX-largest-level-set "section")
      ;; load basic definitons
-     (TeX-run-style-hooks "scrbase")))
+     (TeX-run-style-hooks "scrbase"))
+   LaTeX-dialect)
 
 ;;; scrartcl.el ends here
