@@ -3,6 +3,7 @@
 ;; Copyright (C) 2011 Free Software Foundation, Inc.
 
 ;; Author: Mads Jensen <mje@inducks.org>
+;; Maintainer: auctex-devel@gnu.org
 ;; Created: 2011-02-01
 ;; Keywords: tex
 
@@ -43,7 +44,8 @@
      (font-latex-add-keywords '(("xspace" "")
 				("xspaceaddexception" "{")
 				("xspaceremoveexception" "{"))
-			      'function))))
+			      'function)))
+ LaTeX-dialect)
 
 (defvar LaTeX-xspace-package-options nil
   "Package options for the xspace package.")
