@@ -1,8 +1,9 @@
 ;;; tex-fold.el --- Fold TeX macros.
 
-;; Copyright (C) 2004, 2005 Free Software Foundation, Inc.
+;; Copyright (C) 2004, 2005, 2006, 2007, 2008, 2011-2012
+;;   Free Software Foundation, Inc.
 
-;; Author: Ralf Angeli <angeli@iwi.uni-sb.de>
+;; Author: Ralf Angeli <angeli@caeruleus.net>
 ;; Maintainer: auctex-devel@gnu.org
 ;; Created: 2004-07-04
 ;; Keywords: tex, wp
@@ -11,7 +12,7 @@
 
 ;; AUCTeX is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
+;; the Free Software Foundation; either version 3, or (at your option)
 ;; any later version.
 
 ;; AUCTeX is distributed in the hope that it will be useful, but
@@ -21,8 +22,8 @@
 
 ;; You should have received a copy of the GNU General Public License
 ;; along with AUCTeX; see the file COPYING.  If not, write to the Free
-;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-;; 02111-1307, USA.
+;; Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+;; 02110-1301, USA.
 
 ;;; Commentary:
 
@@ -59,26 +60,75 @@
   "Fold TeX macros."
   :group 'AUCTeX)
 
+(defcustom TeX-fold-type-list '(env macro math)
+  "List of item types to consider when folding.
+Valid items are the symbols 'env for environments, 'macro for
+macros, 'math for math macros and 'comment for comments."
+  :type '(set (const :tag "Environments" env)
+	      (const :tag "Macros" macro)
+	      (const :tag "Math Macros" math)
+	      (const :tag "Comments" comment))
+  :group 'TeX-fold)
+
 (defcustom TeX-fold-macro-spec-list
-  '(("[f]" ("footnote"))
+  `(("[f]" ("footnote" "marginpar"))
     ("[c]" ("cite"))
     ("[l]" ("label"))
-    ("[r]" ("ref"))
-    ("[i]" ("index"))
-    ("*" ("item"))
+    ("[r]" ("ref" "pageref" "eqref"))
+    ("[i]" ("index" "glossary"))
+    ("[1]:||*" ("item"))
     ("..." ("dots"))
+    ("(C)" ("copyright"))
+    ("(R)" ("textregistered"))
+    ("TM"  ("texttrademark"))
     (1 ("part" "chapter" "section" "subsection" "subsubsection"
 	"paragraph" "subparagraph"
 	"part*" "chapter*" "section*" "subsection*" "subsubsection*"
 	"paragraph*" "subparagraph*"
 	"emph" "textit" "textsl" "textmd" "textrm" "textsf" "texttt"
 	"textbf" "textsc" "textup")))
-  "List of display strings and macros to fold."
+  "List of replacement specifiers and macros to fold.
+
+The first element of each item can be a string, an integer or a
+function symbol.  The second element is a list of macros two fold
+without the leading backslash.
+
+If the first element is a string, it will be used as a display
+replacement for the whole macro.  Numbers in braces, brackets,
+parens or angle brackets will be replaced by the respective macro
+argument.  For example \"{1}\" will be replaced by the first
+mandatory argument of the macro.  One can also define
+alternatives within the specifier which are used if an argument
+is not found.  Alternatives are separated by \"||\".  They are
+most useful with optional arguments.  As an example, the default
+specifier for \\item is \"[1]:||*\" which means that if there is
+an optional argument, its value is shown followed by a colon.  If
+there is no optional argument, only an asterisk is used as the
+display string.
+
+If the first element is an integer, the macro will be replaced by
+the respective macro argument.
+
+If the first element is a function symbol, the function will be
+called with all mandatory arguments of the macro and the result
+of the function call will be used as a replacement for the macro.
+
+Setting this variable does not take effect immediately.  Use
+Customize or reset the mode."
   :type '(repeat (group (choice (string :tag "Display String")
-				(integer :tag "Number of argument" :value 1))
+				(integer :tag "Number of argument" :value 1)
+				(function :tag "Function to execute"))
 			(repeat :tag "Macros" (string))))
   :group 'TeX-fold)
 
+(defvar TeX-fold-macro-spec-list-internal nil
+  "Internal list of display strings and macros to fold.
+Is updated when the TeX Fold mode is being activated and then
+contains all constructs to fold for the given buffer or mode
+respectively, i.e. contents of both `TeX-fold-macro-spec-list'
+and <mode-prefix>-fold-macro-spec-list.")
+(make-variable-buffer-local 'TeX-fold-macro-spec-list-internal)
+
 (defcustom TeX-fold-env-spec-list
   '(("[comment]" ("comment")))
   "List of display strings and environments to fold."
@@ -87,6 +137,29 @@
 			(repeat :tag "Environments" (string))))
   :group 'TeX-fold)
 
+(defvar TeX-fold-env-spec-list-internal nil
+  "Internal list of display strings and environments to fold.
+Is updated when the TeX Fold mode is being activated and then
+contains all constructs to fold for the given buffer or mode
+respectively, i.e. contents of both `TeX-fold-env-spec-list'
+and <mode-prefix>-fold-env-spec-list.")
+(make-variable-buffer-local 'TeX-fold-env-spec-list-internal)
+
+(defcustom TeX-fold-math-spec-list nil
+  "List of display strings and math macros to fold."
+  :type '(repeat (group (choice (string :tag "Display String")
+				(integer :tag "Number of argument" :value 1))
+			(repeat :tag "Math Macros" (string))))
+  :group 'TeX-fold)
+
+(defvar TeX-fold-math-spec-list-internal nil
+  "Internal list of display strings and math macros to fold.
+Is updated when the TeX Fold mode is being activated and then
+contains all constructs to fold for the given buffer or mode
+respectively, i.e. contents of both `TeX-fold-math-spec-list'
+and <mode-prefix>-fold-math-spec-list.")
+(make-variable-buffer-local 'TeX-fold-math-spec-list-internal)
+
 (defcustom TeX-fold-unspec-macro-display-string "[m]"
   "Display string for unspecified macros.
 This string will be displayed if a single macro is being hidden
@@ -131,6 +204,11 @@ Set it to zero in order to disable help echos."
   :group 'TeX-fold
   :type 'boolean)
 
+(defcustom TeX-fold-auto nil
+  "If non-nil, fold macros automatically after `TeX-insert-macro'."
+  :group 'TeX-fold
+  :type 'boolean)
+
 (defface TeX-fold-folded-face
   '((((class color) (background light))
      (:foreground "SlateBlue"))
@@ -169,18 +247,26 @@ Set it to zero in order to disable help echos."
 (defvar TeX-fold-open-spots nil)
 (make-variable-buffer-local 'TeX-fold-open-spots)
 
+(defcustom TeX-fold-command-prefix "\C-c\C-o"
+  "Prefix key to use for commands in TeX Fold mode.
+The value of this variable is checked as part of loading TeX Fold mode.
+After that, changing the prefix key requires manipulating keymaps."
+  :type 'string
+  :group 'TeX-fold)
+
 (defvar TeX-fold-keymap
   (let ((map (make-sparse-keymap)))
-    (define-key map "\C-c\C-o\C-o" 'TeX-fold-dwim)
-    (define-key map "\C-c\C-o\C-b" 'TeX-fold-buffer)
-    (define-key map "\C-c\C-o\C-r" 'TeX-fold-region)
-    (define-key map "\C-c\C-o\C-p" 'TeX-fold-paragraph)
-    (define-key map "\C-c\C-o\C-m" 'TeX-fold-macro)
-    (define-key map "\C-c\C-o\C-e" 'TeX-fold-env)
-    (define-key map "\C-c\C-ob"    'TeX-fold-clearout-buffer)
-    (define-key map "\C-c\C-or"    'TeX-fold-clearout-region)
-    (define-key map "\C-c\C-op"    'TeX-fold-clearout-paragraph)
-    (define-key map "\C-c\C-oi"    'TeX-fold-clearout-item)
+    (define-key map "\C-o" 'TeX-fold-dwim)
+    (define-key map "\C-b" 'TeX-fold-buffer)
+    (define-key map "\C-r" 'TeX-fold-region)
+    (define-key map "\C-p" 'TeX-fold-paragraph)
+    (define-key map "\C-m" 'TeX-fold-macro)
+    (define-key map "\C-e" 'TeX-fold-env)
+    (define-key map "\C-c" 'TeX-fold-comment)
+    (define-key map "b"    'TeX-fold-clearout-buffer)
+    (define-key map "r"    'TeX-fold-clearout-region)
+    (define-key map "p"    'TeX-fold-clearout-paragraph)
+    (define-key map "i"    'TeX-fold-clearout-item)
     map))
 
 
@@ -195,12 +281,14 @@ no folded content but a macro or environment, fold it."
   (cond ((TeX-fold-clearout-item))
 	((TeX-active-mark) (TeX-fold-region (mark) (point)))
 	((TeX-fold-item 'macro))
-	((TeX-fold-item 'env))))
+	((TeX-fold-item 'math))
+	((TeX-fold-item 'env))
+	((TeX-fold-comment))))
 
 (defun TeX-fold-buffer ()
   "Hide all configured macros and environments in the current buffer.
 The relevant macros are specified in the variable `TeX-fold-macro-spec-list'
-and environments in `TeX-fold-env-spec-list'."
+and `TeX-fold-math-spec-list', and environments in `TeX-fold-env-spec-list'."
   (interactive)
   (TeX-fold-clearout-region (point-min) (point-max))
   (when (and TeX-fold-force-fontify
@@ -215,7 +303,7 @@ and environments in `TeX-fold-env-spec-list'."
 (defun TeX-fold-paragraph ()
   "Hide all configured macros and environments in the current paragraph.
 The relevant macros are specified in the variable `TeX-fold-macro-spec-list'
-and environments in `TeX-fold-env-spec-list'."
+and `TeX-fold-math-spec-list', and environments in `TeX-fold-env-spec-list'."
   (interactive)
   (save-excursion
     (let ((end (progn (LaTeX-forward-paragraph) (point)))
@@ -223,90 +311,126 @@ and environments in `TeX-fold-env-spec-list'."
       (TeX-fold-clearout-region start end)
       (TeX-fold-region start end))))
 
-(defun TeX-fold-region (start end &optional type)
-  "Fold all items in region starting at position START and ending at END.
-If optional parameter TYPE is given, fold only items of the
-specified type.  TYPE can be one of the symbols 'env for
-environments or 'macro for macros."
+(defun TeX-fold-region (start end)
+  "Fold all items in region from START to END."
   (interactive "r")
-  (if (null type)
-      (progn
-	(TeX-fold-region start end 'env)
-	(TeX-fold-region start end 'macro))
-    (when (or (and (eq type 'env)
-		   (not (eq major-mode 'plain-tex-mode)))
-	      (eq type 'macro))
-      (save-excursion
-	(let (fold-list item-list regexp)
-	  (dolist (item (if (eq type 'env)
-			    TeX-fold-env-spec-list
-			  TeX-fold-macro-spec-list))
-	    (dolist (i (cadr item))
-	      (add-to-list 'fold-list (list i (car item)))
-	      (add-to-list 'item-list i)))
-	  (setq regexp (cond ((and (eq type 'env)
-				   (eq major-mode 'context-mode))
-			      (concat (regexp-quote TeX-esc)
-				      "start" (regexp-opt item-list t)))
-			     ((and (eq type 'env)
-				   (eq major-mode 'texinfo-mode))
-			      (concat (regexp-quote TeX-esc)
-				      (regexp-opt item-list t)))
-			     ((eq type 'env)
-			      (concat (regexp-quote TeX-esc)
-				      "begin[ \t]*{"
-				      (regexp-opt item-list t) "}"))
-			     (t
-			      (concat (regexp-quote TeX-esc)
-				      (regexp-opt item-list t)))))
-	  (save-restriction
-	    (narrow-to-region start end)
-	    ;; Start from the bottom so that it is easier to prioritize
-	    ;; nested macros.
-	    (goto-char (point-max))
-	    (let ((case-fold-search nil)
-		  item-name)
-	      (while (re-search-backward regexp nil t)
-		(setq item-name (match-string 1))
-		(unless (or (and TeX-fold-preserve-comments
-				 (TeX-in-commented-line))
-			    ;; Make sure no partially matched macros are
-			    ;; folded.  For macros consisting of letters
-			    ;; this means there should be none of the
-			    ;; characters [A-Za-z@*] after the matched
-			    ;; string.  Single-char non-letter macros like
-			    ;; \, don't have this requirement.
-			    (and (eq type 'macro)
-				 (save-match-data
-				   (string-match "[A-Za-z]" item-name))
-				 (save-match-data
-				   (string-match "[A-Za-z@*]"
-						 (string (char-after
-							  (match-end 0)))))))
-		  (let* ((item-start (match-beginning 0))
-			 (display-string-spec (cadr (assoc item-name
-							   fold-list)))
-			 (item-end (TeX-fold-item-end item-start type))
-			 (ov (TeX-fold-make-overlay item-start item-end type
-						    display-string-spec)))
-		    (TeX-fold-hide-item ov)))))))))))
+  (when (and (memq 'env TeX-fold-type-list)
+	     (not (eq major-mode 'plain-tex-mode)))
+    (TeX-fold-region-macro-or-env start end 'env))
+  (when (memq 'macro TeX-fold-type-list)
+    (TeX-fold-region-macro-or-env start end 'macro))
+  (when (memq 'math TeX-fold-type-list)
+    (TeX-fold-region-macro-or-env start end 'math))
+  (when (memq 'comment TeX-fold-type-list)
+    (TeX-fold-region-comment start end)))
+
+(defun TeX-fold-region-macro-or-env (start end type)
+  "Fold all items of type TYPE in region from START to END.
+TYPE can be one of the symbols 'env for environments, 'macro
+for macros and 'math for math macros."
+  (save-excursion
+    (let (fold-list item-list regexp)
+      (dolist (item (cond ((eq type 'env) TeX-fold-env-spec-list-internal)
+			  ((eq type 'math) TeX-fold-math-spec-list-internal)
+			  (t TeX-fold-macro-spec-list-internal)))
+	(dolist (i (cadr item))
+	  (add-to-list 'fold-list (list i (car item)))
+	  (add-to-list 'item-list i)))
+      (when item-list
+	(setq regexp (cond ((and (eq type 'env)
+				 (eq major-mode 'context-mode))
+			    (concat (regexp-quote TeX-esc)
+				    "start" (regexp-opt item-list t)))
+			   ((and (eq type 'env)
+				 (eq major-mode 'texinfo-mode))
+			    (concat (regexp-quote TeX-esc)
+				    (regexp-opt item-list t)))
+			   ((eq type 'env)
+			    (concat (regexp-quote TeX-esc)
+				    "begin[ \t]*{"
+				    (regexp-opt item-list t) "}"))
+			   (t
+			    (concat (regexp-quote TeX-esc)
+				    (regexp-opt item-list t)))))
+	(save-restriction
+	  (narrow-to-region start end)
+	  ;; Start from the bottom so that it is easier to prioritize
+	  ;; nested macros.
+	  (goto-char (point-max))
+	  (let ((case-fold-search nil)
+		item-name)
+	    (while (re-search-backward regexp nil t)
+	      (setq item-name (match-string 1))
+	      (unless (or (and TeX-fold-preserve-comments
+			       (TeX-in-commented-line))
+			  ;; Make sure no partially matched macros are
+			  ;; folded.  For macros consisting of letters
+			  ;; this means there should be none of the
+			  ;; characters [A-Za-z@*] after the matched
+			  ;; string.  Single-char non-letter macros like
+			  ;; \, don't have this requirement.
+			  (and (memq type '(macro math))
+			       (save-match-data
+				 (string-match "[A-Za-z]" item-name))
+			       (save-match-data
+				 (string-match "[A-Za-z@*]"
+					       (string (char-after
+							(match-end 0)))))))
+		(let* ((item-start (match-beginning 0))
+		       (display-string-spec (cadr (assoc item-name
+							 fold-list)))
+		       (item-end (TeX-fold-item-end item-start type))
+		       (ov (TeX-fold-make-overlay item-start item-end type
+						  display-string-spec)))
+		  (TeX-fold-hide-item ov))))))))))
+
+(defun TeX-fold-region-comment (start end)
+  "Fold all comments in region from START to END."
+  (save-excursion
+    (goto-char start)
+    (let (beg)
+      (while (setq beg (TeX-search-forward-comment-start end))
+	(goto-char beg)
+	;; Determine the start of the region to be folded just behind
+	;; the comment starter.
+	(looking-at TeX-comment-start-regexp)
+	(setq beg (match-end 0))
+	;; Search for the end of the comment.
+	(while (TeX-comment-forward))
+	(end-of-line 0)
+	;; Hide the whole region.
+	(TeX-fold-hide-item (TeX-fold-make-overlay beg (point) 'comment
+						   TeX-fold-ellipsis))))))
 
 (defun TeX-fold-macro ()
   "Hide the macro on which point currently is located."
   (interactive)
   (unless (TeX-fold-item 'macro)
-    (message "No macro found.")))
+    (message "No macro found")))
+
+(defun TeX-fold-math ()
+  "Hide the math macro on which point currently is located."
+  (interactive)
+  (unless (TeX-fold-item 'math)
+    (message "No macro found")))
 
 (defun TeX-fold-env ()
   "Hide the environment on which point currently is located."
   (interactive)
   (unless (TeX-fold-item 'env)
-    (message "No environment found.")))
+    (message "No environment found")))
+
+(defun TeX-fold-comment ()
+  "Hide the comment on which point currently is located."
+  (interactive)
+  (unless (TeX-fold-comment-do)
+    (message "No comment found")))
 
 (defun TeX-fold-item (type)
   "Hide the item on which point currently is located.
 TYPE specifies the type of item and can be one of the symbols
-'env for environments or 'macro for macros.
+'env for environments, 'macro for macros or 'math for math
+macros.
 Return non-nil if an item was found and folded, nil otherwise."
   (if (and (eq type 'env)
 	   (eq major-mode 'plain-tex-mode))
@@ -349,9 +473,10 @@ Return non-nil if an item was found and folded, nil otherwise."
 			    (if (fboundp 'match-string-no-properties)
 				(match-string-no-properties 1)
 			      (match-string 1))))
-	       (fold-list (if (eq type 'env)
-			      TeX-fold-env-spec-list
-			    TeX-fold-macro-spec-list))
+	       (fold-list (cond ((eq type 'env) TeX-fold-env-spec-list-internal)
+				((eq type 'math)
+				 TeX-fold-math-spec-list-internal)
+				(t TeX-fold-macro-spec-list-internal)))
 	       fold-item
 	       (display-string-spec
 		(or (catch 'found
@@ -371,13 +496,37 @@ Return non-nil if an item was found and folded, nil otherwise."
 					  display-string-spec)))
 	  (TeX-fold-hide-item ov))))))
 
+(defun TeX-fold-comment-do ()
+  "Hide the comment on which point currently is located.
+This is the function doing the work for `TeX-fold-comment'.  It
+is an internal function communicating with return values rather
+than with messages for the user.
+Return non-nil if a comment was found and folded, nil otherwise."
+  (if (and (not (TeX-in-comment)) (not (TeX-in-line-comment)))
+      nil
+    (let (beg)
+      (save-excursion
+	(while (progn
+		 (beginning-of-line 0)
+		 (and (TeX-in-line-comment)
+		      (not (bobp)))))
+	(goto-char (TeX-search-forward-comment-start (line-end-position 2)))
+	(looking-at TeX-comment-start-regexp)
+	(setq beg (match-end 0))
+	(while (TeX-comment-forward))
+	(end-of-line 0)
+	(when (> (point) beg)
+	  (TeX-fold-hide-item (TeX-fold-make-overlay beg (point) 'comment
+						     TeX-fold-ellipsis)))))))
+
 
 ;;; Utilities
 
 (defun TeX-fold-make-overlay (ov-start ov-end type display-string-spec)
   "Make a TeX-fold overlay extending from OV-START to OV-END.
 TYPE is a symbol which is used to describe the content to hide
-and may be 'macro for macros and 'env for environments.
+and may be 'macro for macros, 'math for math macro and 'env for
+environments.
 DISPLAY-STRING-SPEC is the original specification of the display
 string in the variables `TeX-fold-macro-spec-list' or
 `TeX-fold-env-spec-list' and may be a string or an integer."
@@ -394,7 +543,8 @@ string in the variables `TeX-fold-macro-spec-list' or
 
 (defun TeX-fold-item-end (start type)
   "Return the end of an item of type TYPE starting at START.
-TYPE can be either 'env for environments or 'macro for macros."
+TYPE can be either 'env for environments, 'macro for macros or
+'math for math macros."
   (save-excursion
     (cond ((and (eq type 'env)
 		(eq major-mode 'context-mode))
@@ -435,10 +585,14 @@ string DISPLAY-STRING."
 		ov-end))
 	  (current-fill-column))))
 
-(defun TeX-fold-macro-nth-arg (n macro-start &optional macro-end)
+(defun TeX-fold-macro-nth-arg (n macro-start &optional macro-end delims)
   "Return a property list of the argument number N of a macro.
 The start of the macro to examine is given by MACRO-START, its
-end optionally by MACRO-END.
+end optionally by MACRO-END.  With DELIMS the type of delimiters
+can be specified as a cons cell containing the opening char as
+the car and the closing char as the cdr.  The chars have to have
+opening and closing syntax as defined in
+`TeX-search-syntax-table'.
 
 The first item in the returned list is the string specified in
 the argument, the second item may be a face if the argument
@@ -447,20 +601,31 @@ as well, so the second item is always nil.  In XEmacs the string
 does not enclose any faces, so these are given in the second item
 of the resulting list."
   (save-excursion
-    (let ((macro-end (or macro-end
-			 (save-excursion (goto-char macro-start)
-					 (TeX-find-macro-end))))
-	  content-start content-end)
+    (let* ((macro-end (or macro-end
+			  (save-excursion (goto-char macro-start)
+					  (TeX-find-macro-end))))
+	   (open-char (if delims (car delims) ?{))
+	   (open-string (char-to-string open-char))
+	   (close-char (if delims (cdr delims) ?}))
+	   (close-string (char-to-string close-char))
+	   content-start content-end)
       (goto-char macro-start)
       (if (condition-case nil
 	      (progn
 		(while (> n 0)
-		  (skip-chars-forward "^{" macro-end)
-		  (when (not (looking-at "{")) (error nil))
+		  (skip-chars-forward (concat "^" open-string) macro-end)
+		  (when (= (point) macro-end)
+		    (error nil))
 		  (setq content-start (progn
-					(skip-chars-forward "{ \t")
+					(skip-chars-forward
+					 (concat open-string " \t"))
 					(point)))
-		  (goto-char (TeX-find-closing-brace))
+		  (goto-char
+		   (if delims
+		       (with-syntax-table
+			   (TeX-search-syntax-table open-char close-char)
+			 (scan-lists (point) 1 1))
+		     (TeX-find-closing-brace)))
 		  (setq content-end (save-excursion
 				      (backward-char)
 				      (skip-chars-backward " \t")
@@ -594,16 +759,63 @@ Return non-nil if a removal happened, nil otherwise."
 
 ;;; Toggling
 
+(defun TeX-fold-expand-spec (spec ov-start ov-end)
+  "Expand instances of {<num>}, [<num>], <<num>>, and (<num>).
+Replace them with the respective macro argument."
+  (let ((spec-list (split-string spec "||"))
+	(delims '((?{ . ?}) (?[ . ?]) (?< . ?>) (?\( . ?\))))
+	index success)
+    (catch 'success
+      ;; Iterate over alternatives.
+      (dolist (elt spec-list)
+	(setq spec elt
+	      index nil)
+	;; Find and expand every placeholder.
+	(while (and (string-match "\\([[{<]\\)\\([1-9]\\)\\([]}>]\\)" elt index)
+		    ;; Does the closing delim match the opening one?
+		    (string-equal
+		     (match-string 3 elt)
+		     (char-to-string
+		      (cdr (assq (string-to-char (match-string 1 elt))
+				 delims)))))
+	  (setq index (match-end 0))
+	  (let ((arg (car (save-match-data
+			    ;; Get the argument.
+			    (TeX-fold-macro-nth-arg
+			     (string-to-number (match-string 2 elt))
+			     ov-start ov-end
+			     (assoc (string-to-char (match-string 1 elt))
+				    delims))))))
+	    (when arg (setq success t))
+	    ;; Replace the placeholder in the string.
+	    (setq elt (replace-match (or arg TeX-fold-ellipsis) nil t elt)
+		  index (+ index (- (length elt) (length spec)))
+		  spec elt)))
+	(when success (throw 'success nil))))
+    spec))
+
 (defun TeX-fold-hide-item (ov)
   "Hide a single macro or environment.
 That means, put respective properties onto overlay OV."
   (let* ((ov-start (overlay-start ov))
 	 (ov-end (overlay-end ov))
 	 (spec (overlay-get ov 'TeX-fold-display-string-spec))
-	 (computed (if (stringp spec)
-		       spec
-		     (or (TeX-fold-macro-nth-arg spec ov-start ov-end)
-			 "[Error: No content found]")))
+	 (computed (cond
+		    ((stringp spec)
+		     (TeX-fold-expand-spec spec ov-start ov-end))
+		    ((functionp spec)
+		     (let (arg arg-list
+			   (n 1))
+		       (while (setq arg (TeX-fold-macro-nth-arg
+					 n ov-start ov-end))
+			 (add-to-list 'arg-list (car arg) t)
+			 (setq n (1+ n)))
+		       (or (condition-case nil
+			       (apply spec arg-list)
+			     (error nil))
+			   "[Error: No content or function found]")))
+		    (t (or (TeX-fold-macro-nth-arg spec ov-start ov-end)
+			   "[Error: No content found]"))))
 	 (display-string (if (listp computed) (car computed) computed))
 	 (face (when (listp computed) (cadr computed))))
     ;; Cater for zero-length display strings.
@@ -672,9 +884,10 @@ Remove the respective properties from the overlay OV."
 			(and (boundp 'global-disable-point-adjustment)
 			     global-disable-point-adjustment)
 			;; See preview.el on how to make this configurable.
-			(memq this-command (list (key-binding [left])
-						 (key-binding [right])
-						 'mouse-set-point)))
+			(memq this-command
+			      (list (key-binding [left]) (key-binding [right])
+				    'backward-char 'forward-char
+				    'mouse-set-point)))
 		;; Open new overlays.
 		(dolist (ol (nconc (when (and TeX-fold-unfold-around-mark
 					      (boundp 'mark-active)
@@ -705,7 +918,7 @@ Remove the respective properties from the overlay OV."
 ;;; Misc
 
 ;; Copy and adaption of `cvs-partition' from pcvs-util.el in GNU Emacs
-;; on 2004-07-05 to make latex-fold.el mainly self-contained.
+;; on 2004-07-05 to make tex-fold.el mainly self-contained.
 (defun TeX-fold-partition-list (p l)
   "Partition a list L into two lists based on predicate P.
 The function returns a `cons' cell where the `car' contains
@@ -719,24 +932,43 @@ the other elements.  The ordering among elements is maintained."
 
 ;;; The mode
 
-;;; This autoload cookie had to be changed because of XEmacs.  This is
-;;; very dissatisfactory, because we now don't have the full doc string
-;;; available to tell people what to expect when using this mode
-;;; before loading it.
+;; This autoload cookie had to be changed because of XEmacs.  This is
+;; very dissatisfactory, because we now don't have the full doc string
+;; available to tell people what to expect when using this mode before
+;; loading it.
 
-;;;###autoload (autoload 'TeX-fold-mode "tex-fold" "Minor mode for hiding and revealing macros and environments.")
+;;;###autoload (autoload 'TeX-fold-mode "tex-fold" "Minor mode for hiding and revealing macros and environments." t)
 (define-minor-mode TeX-fold-mode
   "Minor mode for hiding and revealing macros and environments.
 
 Called interactively, with no prefix argument, toggle the mode.
 With universal prefix ARG (or if ARG is nil) turn mode on.
 With zero or negative ARG turn mode off."
-  nil nil TeX-fold-keymap
+  nil nil (list (cons TeX-fold-command-prefix TeX-fold-keymap))
   (if TeX-fold-mode
       (progn
 	(set (make-local-variable 'search-invisible) t)
 	(add-hook 'post-command-hook 'TeX-fold-post-command nil t)
-	(add-hook 'LaTeX-fill-newline-hook 'TeX-fold-update-at-point nil t))
+	(add-hook 'LaTeX-fill-newline-hook 'TeX-fold-update-at-point nil t)
+	(add-hook 'TeX-after-insert-macro-hook
+		  (lambda ()
+		    (when (and TeX-fold-mode TeX-fold-auto)
+		      (save-excursion
+			(backward-char)
+			(or (TeX-fold-item 'macro)
+			    (TeX-fold-item 'math)
+			    (TeX-fold-item 'env))))))
+	;; Update the `TeX-fold-*-spec-list-internal' variables.
+	(dolist (elt '("macro" "env" "math"))
+	  (set (intern (format "TeX-fold-%s-spec-list-internal" elt))
+	       ;; Append the value of `TeX-fold-*-spec-list' to the
+	       ;; mode-specific `<mode-prefix>-fold-*-spec-list' variable.
+	       (append (symbol-value (intern (format "TeX-fold-%s-spec-list"
+						     elt)))
+		       (let ((symbol (intern (format "%s-fold-%s-spec-list"
+						     (TeX-mode-prefix) elt))))
+			 (when (boundp symbol)
+			   (symbol-value symbol)))))))
     (kill-local-variable 'search-invisible)
     (remove-hook 'post-command-hook 'TeX-fold-post-command t)
     (remove-hook 'LaTeX-fill-newline-hook 'TeX-fold-update-at-point t)
